package lab.game;

import java.io.Serial;

import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;
import javafx.scene.paint.Color;

public class Background extends WorldEntity {

	@Serial
	private static final long serialVersionUID = 2566936061563448971L;
	
	private MyPoint[] imagePosition;
	private static Image[] image;
	private MyPoint speed;
	private MyDimension sceneSize; 
	private static final double[] speedMultiplier = new double[] { 0.6, 0.8, 1, 1 };

	public Background(Scene scene) {
		super(scene, new MyPoint(0, 0), 0);
		sceneSize = scene.getSize();
		imagePosition = new MyPoint[4];
		imagePosition[0] = new MyPoint(0, getImage()[0].getHeight() - sceneSize.getHeight());
		imagePosition[1] = new MyPoint(0, getImage()[1].getHeight() - sceneSize.getHeight());
		imagePosition[2] = new MyPoint(0, getImage()[2].getHeight() - sceneSize.getHeight());
		imagePosition[3] = new MyPoint(0, getImage()[3].getHeight() - sceneSize.getHeight());
		speed = new MyPoint(100, 0);
	}

	private static Image[] getImage() {
		if(image == null) {
			image = new Image[4];
			image[0] = new Image(Background.class.getResourceAsStream("cityfar400.png"));
			image[1] = new Image(Background.class.getResourceAsStream("citymid400.png"));
			image[2] = new Image(Background.class.getResourceAsStream("cityclose400.png"));
			image[3] = new Image(Background.class.getResourceAsStream("cityreflection400.png"));
		}
		return image;
	}

	@Override
	public void drawInternal(GraphicsContext gc) {
		gc.setFill(Color.DARKBLUE);
		gc.fillRect(0, 0, sceneSize.getWidth(), sceneSize.getHeight());
		for (int i = 0; i < getImage().length; i++) {
			double w = sceneSize.getWidth();
			double x = imagePosition[i].getX();
			double realWidth = x + w <= getImage()[i].getWidth() ? w : getImage()[i].getWidth() - x;
			gc.drawImage(getImage()[i], x, imagePosition[i].getY(), realWidth, sceneSize.getHeight(), 0, 0, realWidth,
					sceneSize.getHeight());
			if (x + w > getImage()[i].getWidth()) {
				gc.drawImage(getImage()[i], 0, imagePosition[i].getY(), w - realWidth, sceneSize.getHeight(),
						getImage()[i].getWidth() - imagePosition[i].getX(), 0, w - realWidth, sceneSize.getHeight());
			}
		}
	}

	@Override
	public void simulate(double deltaTime) {
		MyPoint baseSpeed = speed.multiply(deltaTime / 1_000_000_000);
		for (int i = 0; i < getImage().length; i++) {
			imagePosition[i] = imagePosition[i].add(baseSpeed.multiply(speedMultiplier[i]));
			if (imagePosition[i].getX() > getImage()[i].getWidth()) {
				imagePosition[i] = new MyPoint(0, imagePosition[i].getY());
			}
		}
	}

	public void setSpeed(double doubleValue) {
		speed = new MyPoint(doubleValue, 0);
	}
}
