package lab.game;

import java.io.Serial;
import java.util.Random;

import javafx.geometry.Rectangle2D;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;
import lab.Setup;

public class Boat extends WorldEntity implements Collisionable {

	@Serial
	private static final long serialVersionUID = -8233882073179438194L;
	
	private static final Random RANDOM = new Random();
	private MyPoint speed;
	private static Image image;

	public Boat(Scene scene, MyPoint position) {
		super(scene, position, 100);
		speed = new MyPoint(0, 0);
	}

	private static Image getImage() {
		if(image == null) {
			image = new Image(Boat.class.getResourceAsStream("ship-boat.gif"));
		}
		return image;
	}

	@Override
	public void drawInternal(GraphicsContext gc) {
		gc.drawImage(getImage(), position.getX(), position.getY());
		Rectangle2D rec = getBoundingBox();
		gc.strokeRect(rec.getMinX(), rec.getMinY(), rec.getWidth(), rec.getHeight());
	}

	@Override
	public void simulate(double deltaTime) {
		position = position.add(speed.multiply(deltaTime / 1_000_000_000));
		speed = speed.multiply(0.99);
	}

	@Override
	public Rectangle2D getBoundingBox() {
		return new Rectangle2D(position.getX(),
				position.getY() + getImage().getHeight() * (1 - Setup.getInstance().getBoatCollisionHeight()),
				getImage().getWidth(), getImage().getHeight() * Setup.getInstance().getBoatCollisionHeight());
	}

	@Override
	public boolean intersect(Rectangle2D another) {
		return getBoundingBox().intersects(another);
	}

	@Override
	public void hitBy(Collisionable another) {
		if (another instanceof LochNess lochNess) {
			int direction = RANDOM.nextBoolean() ? -1 : 1;
			speed = new MyPoint(-Setup.getInstance().getBoatHitPulseX(),
					direction * RANDOM.nextDouble(Setup.getInstance().getBoatHitPulseYMin(),
							Setup.getInstance().getBoatHitPulseYMax()));
		}
	}

	public void setPosInPercentage(double doubleValue) {
		double colisionHeight = getBoundingBox().getHeight();
		double posFromBottom = (scene.getSize().getHeight()-colisionHeight)*doubleValue/100;
		position = new MyPoint(position.getX(),
				(scene.getSize().getHeight()-getImage().getHeight() -posFromBottom));
	}

}
