package lab.gui;

import java.io.IOException;
import java.net.URL;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonType;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.stage.WindowEvent;
import lab.Setup;
import lab.data.Score;
import lab.storage.FileStorage;

/**
 * Class <b>App</b> - extends class Application and it is an entry point of the
 * program
 * 
 * @author Java I
 */
public class App extends Application {

	private static Logger log = LogManager.getLogger(App.class);
	private GameController gameController;
	private MainScreenController menuController;
	private boolean viewMode;

	private Stage primaryStage;

	public static void main(String[] args) {
		log.info("Application lauched");
		Setup.configure(Setup.builder().scoreStorageInterface(new FileStorage()).build());

		launch(args);
	}

	@Override
	public void start(Stage primaryStage) {
		try {
			Alert alert = new Alert(AlertType.CONFIRMATION, "View Mode?", ButtonType.YES, ButtonType.NO);
			alert.showAndWait().ifPresent(button -> viewMode = ButtonType.YES.equals(button));

			this.primaryStage = primaryStage;
			switchToMenu();
			primaryStage.setTitle("Java 2 - 2th laboratory");
			primaryStage.show();
			// Exit program when main window is closed
			primaryStage.setOnCloseRequest(this::exitProgram);
		} catch (Exception e) {
			log.error("Error during game play.", e);
		}
	}

	public void switchToGame(String name, int numberOfMonsters) throws IOException {
		// Construct a main window with a canvas.
		FXMLLoader gameLoader = new FXMLLoader(getClass().getResource("/lab/gui/gameWindow.fxml"));
		Parent root = gameLoader.load();
		gameController = gameLoader.getController();
		gameController.setViewMode(viewMode);
		Scene scene = new Scene(root);
		URL cssUrl = getClass().getResource("application.css");
		scene.getStylesheets().add(cssUrl.toString());
		primaryStage.setScene(scene);
		gameController.startGame(name, numberOfMonsters);
	}

	private void switchToMenu() throws IOException {
		// Construct a main window with a canvas.
		FXMLLoader menuLoader = new FXMLLoader(getClass().getResource("/lab/gui/mainScreen.fxml"));
		Parent root = menuLoader.load();
		menuController = menuLoader.getController();
		menuController.setApp(this);
		Scene scene = new Scene(root);
		URL cssUrl = getClass().getResource("application.css");
		scene.getStylesheets().add(cssUrl.toString());
		primaryStage.setScene(scene);
	}
	
	public Stage createDialogStage(Score score) throws IOException {
		Stage dialog = new Stage();
		dialog.initStyle(StageStyle.UTILITY);
		FXMLLoader editLoader = new FXMLLoader(getClass().getResource("/lab/gui/edit.fxml"));
		Parent root = editLoader.load();
		EditController editController = editLoader.getController();
		editController.setStage(dialog);
		editController.setObjectToEdit(score);
		editController.setMainScreenController(menuController);
		Scene scene = new Scene(root);
		URL cssUrl = getClass().getResource("application.css");
		scene.getStylesheets().add(cssUrl.toString());
		dialog.setScene(scene);
		dialog.initModality(Modality.APPLICATION_MODAL);
		dialog.initOwner(primaryStage);
		return dialog;

	}

	@Override
	public void stop() throws Exception {
		if (gameController != null) {
			gameController.stop();
		}
		super.stop();
		log.info("Gamne stoped");
	}

	private void exitProgram(WindowEvent evt) {
		if (gameController != null) {
			gameController.stop();
		}
		Setup.getInstance().getScoreStorageInterface().stop();
		log.info("Exiting game");
		System.exit(0);
	}
}