package lab.storage;

import java.util.List;
import java.util.function.Consumer;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.Persistence;
import lab.data.Score;

public class JpaConnector implements ScoreStorageInterface {


	private EntityManagerFactory entityManagerFactory;
	private EntityManager entityManager;
	
	public JpaConnector() {
		entityManagerFactory = Persistence.createEntityManagerFactory("java2");
		entityManager = entityManagerFactory.createEntityManager();
	}

	@Override
	public List<Score> getAll() {
		return entityManager.createQuery("select s from Score s", Score.class).getResultList();
	}

	@Override
	public List<Score> getFirstTen() {
		return entityManager.createQuery("select s from Score s order by s.points DESC", Score.class).setMaxResults(10).getResultList();
	}

	@Override
	public void init() {
	}

	@Override
	public Score save(Score score) {
		Score result;
		entityManager.getTransaction().begin();
		if(score.getId() == null || score.getId() == 0) {
			entityManager.persist(score);
			result = score;
		} else {
			result = entityManager.merge(score);
		}
		entityManager.getTransaction().commit();
		return result;
	}

	@Override
	public void delete(List<Score> scores) {
		entityManager.getTransaction().begin();
		for (Score score : scores) {
			entityManager.remove(score);
		}
		entityManager.getTransaction().commit();
	}

	@Override
	public void stop() {
		entityManager.close();
		entityManagerFactory.close();
	}

	public Object getEntityManager() {
		//return entity manager. Type Object is there because of compilation of empty task assignment
		return entityManager;
	}

	public Score find(long id) {
		return entityManager.find(Score.class, id);
	}
	
	public void modifyNoPersistOrMerge(long id, Consumer<Score> motificator) {
		entityManager.getTransaction().begin();
		Score score = find(id);
		motificator.accept(score);
		entityManager.getTransaction().commit();
	}
}
