package jez04.structure.test;

import static org.hamcrest.MatcherAssert.assertThat;

import java.net.URISyntaxException;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.hamcrest.Matchers;
import org.junit.jupiter.api.Test;

import cz.vsb.fei.kelvin.unittest.ProjectContains;
import cz.vsb.fei.kelvin.unittest.StructureHelper;
import cz.vsb.fei.kelvin.unittest.TextFileContains;
import cz.vsb.fei.kelvin.unittest.XmlFileContains;

class ClassStructureTestHw {

	StructureHelper helper = StructureHelper.getInstance(ClassStructureTest.class);

	@Test
	void jakartaAndHibernateAsDependencyTest() throws URISyntaxException {
		assertThat(TextFileContains.getProjectRoot(getClass()), new XmlFileContains("pom.xml",
				"/project/dependencies/dependency/artifactId[text() = 'jakarta.persistence-api']"));
		assertThat(TextFileContains.getProjectRoot(getClass()), new XmlFileContains("pom.xml",
				"/project/dependencies/dependency/artifactId[text() = 'hibernate-core']"));
	}

	@Test
	void jakartaInModulInfoTest() throws URISyntaxException {
		assertThat(TextFileContains.getProjectRoot(getClass()),
				new TextFileContains("module-info.java", "jakarta.persistence;"));
		assertThat(TextFileContains.getProjectRoot(getClass()),
				new TextFileContains("module-info.java", "opens\\s+lab.data;|opens\\s+lab.data\\s+to\\s+javafx.base,org.hibernate.orm.core;"));

	}

	@Test
	void pesistenceXmlTest() throws URISyntaxException {
		ProjectContains projectContains = new ProjectContains("persistence.xml");
		assertThat(TextFileContains.getProjectRoot(getClass()), projectContains);
		assertThat(projectContains.getFoundFiles(), Matchers.not(Matchers.hasSize(0)));
		Path persistenceXml = projectContains.getFoundFiles().getFirst();
		assertThat(persistenceXml.toAbsolutePath().toString(),
				Matchers.endsWith(Paths.get("resources", "META-INF", "persistence.xml").toString()));
	}

	@Test
	void useEntityTest() throws URISyntaxException {
		assertThat(TextFileContains.getProjectRoot(getClass()),
				new TextFileContains(".*\\.java", "@Entity").useRegExpForName(true));
	}

	@Test
	void useIdTest() throws URISyntaxException {
		assertThat(TextFileContains.getProjectRoot(getClass()),
				new TextFileContains(".*\\.java", "@Id").useRegExpForName(true));
	}

	@Test
	void useGeneratedValueTest() throws URISyntaxException {
		assertThat(TextFileContains.getProjectRoot(getClass()),
				new TextFileContains(".*\\.java", "@GeneratedValue").useRegExpForName(true));
	}

	@Test
	void useEntityManagerTest() throws URISyntaxException {
		assertThat(TextFileContains.getProjectRoot(getClass()),
				new TextFileContains(".*\\.java", "EntityManager").useRegExpForName(true));
	}

	@Test
	void usePersistOrMergeTest() throws URISyntaxException {
		assertThat(TextFileContains.getProjectRoot(getClass()),
				new TextFileContains(".*\\.java", "\\.persist\\(|\\.merge\\(").useRegExpForName(true));
	}
}
